import aiohttp, asyncio, requests, json, aiofiles

from .errors import *

def fix_format(string):
    keys = {
        " ": "+",
        "/": "\/",
        "'": "\'",
        '"': '\"'
    }

    for key in keys:
        string.replace(key, keys[key])
    return string

class AsyncApp:
    def __init__(self, appID):
        self.id = appID
        self.SIMPLE_BASE = "https://api.wolframalpha.com/v1/simple"
        self.FULL_BASE = "https://api.wolframalpha.com/v2/query"
        self.SHORT_BASE = "http://api.wolframalpha.com/v1/result"
        self.CONV_BASE = "http://api.wolframalpha.com/v1/conversation.jsp"

    async def create_request(self, BASE, **kwargs):
        url = BASE + "?" + '&'.join(f"{i}={kwargs[i]}" for i in kwargs) + "&appid=" + self.id

        async with aiohttp.ClientSession() as session:
            async with session.get(url) as request:
                return await request.text()
    
    async def create_simple_request(self, **kwargs):
        async with aiohttp.ClientSession() as session:
            url = self.SIMPLE_BASE + "?" + '&'.join(f"{i}={kwargs[i]}" for i in kwargs) + "&appid=" + self.id
            async with session.get(url) as request:
                return await request.read()

    async def simple(self, query:lambda arg:fix_format(arg), fp:str="wolfram_content"):
        _data = await self.create_simple_request(i=query)

        f = await aiofiles.open(f'{fp}.png', mode='wb+')
        await f.write(_data)
        await f.close()

    async def full(self, query:lambda arg:fix_format(arg)):
        _data = await self.create_request(self.FULL_BASE, input=fix_format(query), output="json")
        return json.loads(_data)

    async def short(self, query:lambda arg:fix_format(arg)):
        return await self.create_request(self.SHORT_BASE, i=query)

    async def talk(self, query:lambda arg:fix_format(arg)):
        _data = await self.create_request(self.CONV_BASE, i=query, s=5)
        return json.loads(_data)


import requests

from .errors import *

def fix_format(string):
    keys = {
        " ": "+",
        "/": "\/",
        "'": "\'",
        '"': '\"'
    }

    for key in keys:
        string.replace(key, keys[key])
    return string

class App:
    def __init__(self, appID):
        self.id = appID
        self.SIMPLE_BASE = "https://api.wolframalpha.com/v1/simple"
        self.FULL_BASE = "https://api.wolframalpha.com/v2/query"
        self.SHORT_BASE = "http://api.wolframalpha.com/v1/result"
        self.CONV_BASE = "http://api.wolframalpha.com/v1/conversation.jsp"

    def create_request(self, BASE, **kwargs):
        return requests.get(BASE + "?" + '&'.join(f"{i}={kwargs[i]}" for i in kwargs) + "&appid=" + self.id)

    def simple(self, query:lambda arg:fix_format(arg), fp:lambda arg:str(arg)="wolframpy_content"):
        _data = self.create_request(self.SIMPLE_BASE, i=query)

        if _data.status_code == 200:
            with open(fp+".png", "wb") as f:
                for chunk in _data.iter_content(1024):
                    f.write(chunk)
        
        if _data.status_code == 404:
            raise APIError("The WolframAPI is currently unreachable!")
        
        if _data.status_code == 501:
            raise InputError("Could not understand input.")

    def full(self, query:lambda arg:fix_format(arg)):
        return self.create_request(self.FULL_BASE, input=query, output="json").json()

    def short(self, query:lambda arg:fix_format(arg)):
        return self.create_request(self.SHORT_BASE, i=query).text

    def talk(self, query:lambda arg:fix_format(arg)):
        return self.create_request(self.CONV_BASE, i=query, s=5).json()


from setuptools import setup

with open("README.md", "r") as f:
  readme = f.read()

setup(
  name = 'WolframAlpha.py',
  packages = ['wolfram'],
  version = '1.1.5',
  license='MIT',
  description = 'A WolframAlpha API Wrapper for Python.',
  author = 'Nemika',
  author_email = 'nemika@bytestobits.dev',
  url = 'https://github.com/Nemika-Haj/WolframAlpha.py',
  keywords = ["wolfram", "wolframalpha", "wolfram.py", "wolframalpha.py", "wrapper"],
  long_description=readme,
  long_description_content_type="text/markdown",
  install_requires=[
          'requests',
          'aiofiles'
      ],
  classifiers=[
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9'
  ],
)